package com.doverfuelingsolutions.issp.api

import com.doverfuelingsolutions.issp.R
import com.doverfuelingsolutions.issp.api.basic.RetrofitUtil
import com.doverfuelingsolutions.issp.api.dto.DFSResult
import com.doverfuelingsolutions.issp.api.dto.ResultPumpInfo
import com.doverfuelingsolutions.issp.api.entity.NozzleInfo
import com.doverfuelingsolutions.issp.api.entity.PumpInfo
import com.doverfuelingsolutions.issp.api.service.ServiceStation
import com.doverfuelingsolutions.issp.utils.log.DFSLog
import com.doverfuelingsolutions.issp.utils.sp.SPKeys
import com.doverfuelingsolutions.issp.utils.sp.SPUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import kotlin.coroutines.resume
import kotlin.coroutines.suspendCoroutine

object FuelInfoApi {

    private var serviceStation: ServiceStation = serviceMaker()

    suspend fun requestPumpInfo() = suspendCoroutine<DFSResult<List<PumpInfo>>> {
        GlobalScope.launch(Dispatchers.IO) {
            val resultPumpList = requestPumpList()
            if (!resultPumpList.success || resultPumpList.data == null) {
                it.resume(DFSResult.fail(resultPumpList.message))
                return@launch
            } else if (resultPumpList.data.isEmpty()) {
                it.resume(DFSResult.success(emptyList()))
                return@launch
            }

            val localBarcodeMap = getLocalBarcodeMap().toMutableMap()
            var isLocalAdd = false
            val pumpList = arrayListOf<PumpInfo>()
            for (item in resultPumpList.data) {
                if (item.nozzles.isEmpty()) return@launch
                // confirm 确认是否每个油泵下的枪的油品都是一致的？ResultPumpInfo 的 Name 属性？
                // 当前油泵的油品名称
                val firstNozzle = item.nozzles[0]
                val localName = localBarcodeMap[firstNozzle.ProductBarcode]
                val name: String = if (localName != null && localName.isNotEmpty()) {
                    localName
                } else {
                    val resultBarcodeInfo = CloudApi.barcodeName(firstNozzle.ProductBarcode)
                    if (resultBarcodeInfo.success && resultBarcodeInfo.data != null && resultBarcodeInfo.data.IsFuelItem) {
                        localBarcodeMap[firstNozzle.ProductBarcode] = resultBarcodeInfo.data.ItemName
                        isLocalAdd = true
                        resultBarcodeInfo.data.ItemName
                    } else {
                        ""
                    }
                }
                val pump = PumpInfo(item.PumpId, name, item.nozzles.map { resultNozzleInfo ->
                    NozzleInfo(item.PumpId, resultNozzleInfo.SiteLevelNozzleId, resultNozzleInfo.ProductBarcode, name)
                })
                pumpList.add(pump)
            }
            if (isLocalAdd) setLocalBarcodeMap(localBarcodeMap)

            // confirm 是否还有必要做排序？
            pumpList.sortBy { it.pumpId }
            it.resume(DFSResult.success(pumpList))
        }
    }

    // confirm 订单主键从 nozzle 改为 pump
    private suspend fun requestPumpList() = suspendCoroutine<DFSResult<Array<ResultPumpInfo>>> {
        serviceStation.getNozzleInfo().enqueue(object : Callback<Array<ResultPumpInfo>> {
            override fun onResponse(call: Call<Array<ResultPumpInfo>>, response: Response<Array<ResultPumpInfo>>) {
                val code = response.code()
                val body = response.body()
                if (code == 200 && body != null) {
                    it.resume(DFSResult.success(body))
                    return
                }
                it.resume(DFSResult.fail(R.string.return_data_error))
            }

            override fun onFailure(call: Call<Array<ResultPumpInfo>>, t: Throwable) {
                DFSLog.e("FuelInfoApi.getNozzleData.onFailure", t)
                it.resume(DFSResult.fail(t.message.toString()))
            }
        })
    }

    private fun getLocalBarcodeMap(): Map<Int, String> {
        return mutableMapOf<Int, String>().apply {
            val localBarcodeList = SPUtil.getString(SPKeys.FUEL_BARCODE)
            localBarcodeList.split(",").forEach {
                val kv = it.split("=")
                if (kv.size == 2) put(kv[0].toInt(), kv[1])
            }
        }
    }

    private fun setLocalBarcodeMap(map: Map<Int, String>) {
        SPUtil.putString(SPKeys.FUEL_BARCODE, map.entries.joinToString(",") { "${it.key}=${it.value}" })
    }

    private fun serviceMaker(): ServiceStation {
        val ip = SPUtil.getString(SPKeys.FUEL_IP)
        val port = SPUtil.getString(SPKeys.FUEL_PORT)
        val url = "http://$ip:$port"
        return RetrofitUtil.getGsonBuilder().baseUrl(url).build().create(ServiceStation::class.java)
    }

    fun generateService() {
        serviceStation = serviceMaker()
    }
}