package com.doverfuelingsolutions.issp.view

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.View
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentResultListener
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.lifecycleScope
import com.doverfuelingsolutions.issp.R
import com.doverfuelingsolutions.issp.data.GlobalData
import com.doverfuelingsolutions.issp.databinding.ActivityMainBinding
import com.doverfuelingsolutions.issp.fusion.FusionError
import com.doverfuelingsolutions.issp.fusion.FusionManager
import com.doverfuelingsolutions.issp.fusion.callback.OnFusionEvent
import com.doverfuelingsolutions.issp.utils.ActivityUtil
import com.doverfuelingsolutions.issp.utils.StringUtil
import com.doverfuelingsolutions.issp.view.fragment.*
import com.wayne.www.waynelib.fdc.FdcClient
import com.youth.banner.adapter.BannerImageAdapter
import com.youth.banner.holder.BannerImageHolder
import kotlinx.coroutines.launch
import java.io.File

class MainActivity : AppCompatActivity(),
    OnFusionEvent,
    View.OnLongClickListener,
    FragmentResultListener {

    companion object {
        fun start(context: Context) {
            Intent(context, MainActivity::class.java).let { context.startActivity(it) }
        }
    }

    private val binding: ActivityMainBinding by lazy { DataBindingUtil.setContentView(this, R.layout.activity_main) }
    private val mainViewModel: MainViewModel by viewModels()

    private val fragmentHolder = FragmentHolder()
    private val fragmentSelect = FragmentSelect()
    private val fragmentNozzle: FragmentNozzle by lazyOf(FragmentNozzle())
    private val fragmentOrderList: FragmentOrderList by lazyOf(FragmentOrderList())
    private val fragmentPayType: FragmentPayType by lazyOf(FragmentPayType())
    private var isFragmentHoldBack = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding.lifecycleOwner = this
        binding.mainViewModel = mainViewModel
        initView()
        initFusion()


    }

    override fun onResume() {
        super.onResume()
        ActivityUtil.setFullscreen(this)
    }

    override fun onDestroy() {
        super.onDestroy()

        FusionManager.onFusionEvent = null
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        lifecycleScope.launch {
            when (requestCode) {
                PreferenceActivity.codeRequestResult -> {
                    data?.let {
                        val isMiddleModified = it.getBooleanExtra(PreferenceActivity.isMiddleModified, false)
                        val isFuelModified = it.getBooleanExtra(PreferenceActivity.isFuelModified, false)
                        if (!isFuelModified && !isMiddleModified) return@let

                        setFragment(fragmentHolder)
                        if (isMiddleModified || FusionManager.stateFusion != FdcClient.FdcClientState.Connected) {
                            fragmentHolder.loading(StringUtil.get(R.string.in_reconnect_fusion))
                            FusionManager.restart()
                        } else if (isFuelModified && FusionManager.stateFusion == FdcClient.FdcClientState.Connected) {
                            fragmentHolder.loading(StringUtil.get(R.string.in_get_fuel))
                            FusionManager.loginFetchInfo()
                        }
                    }
                }
            }
        }
    }

    override fun onBackPressed() {
        if (isFragmentHoldBack) super.onBackPressed()
    }

    override fun onFragmentResult(requestKey: String, result: Bundle) {
        if (requestKey != FragmentConstants.fragmentEvent) return

        when (result.getString(FragmentConstants.eventType)) {
            FragmentConstants.eventHome -> {
                if (supportFragmentManager.backStackEntryCount > 0)
                    supportFragmentManager.popBackStack(supportFragmentManager.getBackStackEntryAt(0).id, FragmentManager.POP_BACK_STACK_INCLUSIVE)
            }
            FragmentConstants.eventBack -> {
                supportFragmentManager.popBackStack()
            }
            FragmentConstants.eventSearchBy -> {
                val type = result.getString(FragmentConstants.searchType)
                if (type == FragmentConstants.typeNozzle) {
                    setFragment(fragmentNozzle, true)
                } else if (type == FragmentConstants.typeOilName) {
                    setFragment(fragmentNozzle, true)
                }
            }
            FragmentConstants.eventOrderList -> {
                val type = result.getString(FragmentConstants.searchType)
                if (type == FragmentConstants.typeNozzle) {
                    fragmentOrderList.viewModel.type.value = FragmentConstants.typeNozzle
                    fragmentOrderList.viewModel.pump.value = result.getInt(FragmentConstants.pumpId)
                    setFragment(fragmentOrderList, true)
                }
            }
        }
    }

    override fun onLongClick(v: View?): Boolean {
        when (v) {
            binding.clock -> {
                PreferenceActivity.startForResult(this)
            }
        }
        return false
    }

    override fun onFusionInit(code: FusionError, msg: String) {
        lifecycleScope.launch {
            // WrongAddress Timeout 需要修改中控信息，返回时如已修改则重启Fusion
            // GetNozzleInfo 不重启Fusion，重新请求加油信息即可
            when (code) {
                FusionError.Success -> {
                    setFragment(fragmentSelect)
                    // Preload order data after first connect
                    lifecycleScope.launch {
                        FusionManager.getAllUnsettledOrder(-1).run {
                            if (success && data != null) {
                                GlobalData.orderList.set(data.toMutableList())
                            }
                        }
                    }
                }
                FusionError.WrongAddress, FusionError.Timeout, FusionError.GetNozzleInfo -> {
                    setFragment(fragmentHolder)
                    val buttonText = if (code == FusionError.GetNozzleInfo) {
                        StringUtil.get(R.string.go_check_fuel)
                    } else {
                        StringUtil.get(R.string.go_check_fusion)
                    }
                    fragmentHolder.action(StringUtil.get(R.string.connect_fail_reason, msg), buttonText) {
                        PreferenceActivity.startForResult(this@MainActivity)
                    }
                }
                FusionError.Login -> {
                    setFragment(fragmentHolder)
                    fragmentHolder.action(StringUtil.get(R.string.connect_fail_reason, msg), StringUtil.get(R.string.retry_login)) {
                        FusionManager.loginFetchInfo()
                    }
                }
            }
        }
    }

    override fun onFusionReconnect() {
        lifecycleScope.launch {
            setFragment(fragmentHolder)
            fragmentHolder.loading(StringUtil.get(R.string.in_reconnect_fusion))
        }
    }

    private fun initView() {
        supportActionBar?.hide()
        mainViewModel.stationName.value = GlobalData.businessName.get()
        if (GlobalData.stationLogoFileName.get().isNotEmpty())
            binding.stationLogo.setImageURI(Uri.fromFile(File(filesDir, GlobalData.stationLogoFileName.get())))
        binding.banner.run {
            addBannerLifecycleObserver(this@MainActivity)
            scrollTime = 500
            adapter =
                object : BannerImageAdapter<Int>(listOf(R.mipmap.banner_1, R.mipmap.banner_2)) {
                    override fun onBindView(holder: BannerImageHolder, data: Int, position: Int, size: Int) {
                        holder.imageView.setImageResource(data)
                    }
                }
        }
        binding.clock.setOnLongClickListener(this)

        supportFragmentManager.setFragmentResultListener(FragmentConstants.fragmentEvent, this, this)
        setFragment(fragmentPayType)
    }

    private fun initFusion() {
        FusionManager.onFusionEvent = this
        // lifecycle.addObserver(FusionManager)
    }

    private fun setFragment(fragment: Fragment, back: Boolean = false) {
        val contain = supportFragmentManager.fragments.contains(fragment)
        if (contain) return

        supportFragmentManager.beginTransaction().run {
            replace(R.id.fragmentBox, fragment)
            if (back) {
                addToBackStack(fragment.javaClass.simpleName)
            }
            commit()
        }
    }

    class MainViewModel : ViewModel() {
        val stationName = MutableLiveData("")
    }
}